import asyncio
import time
from asyncio import Semaphore
from random import random
from typing import Iterable, List, Union

from aiohttp import ClientSession
from asgiref.sync import async_to_sync
from django.core.management import BaseCommand
from django.utils import timezone

from appel_crises.data import DISTRICT_TO_MP
from appel_crises.data.gvt import GVT
from appel_crises.mailer import (
    send_mail,
    MAIN_PERSON,
    mailing_person,
    get_aiohttp_session,
    MailingPerson,
)
from appel_crises.models import Signature, CallOutEmail
from appel_crises.settings import MAIN_EMAIL, APPMODE

MAX_CONCURRENT_REQUESTS = 100
SendSignature = Union[Signature, type(None)]
SendCallOutEmail = Union[CallOutEmail, type(None)]


def get_gvt_address() -> MailingPerson:
    """Returns an address from the government, as specified in data.gvt ."""
    return random.choice(GVT)


def mailing_person_from_circonscription(circo_number: str) -> MailingPerson:
    """Fetch the mailing person from stored data."""

    mp = DISTRICT_TO_MP[circo_number]

    if APPMODE != 'prod':
        mailto = "hadrien.renaudlebret@gmail.com"
    else:
        mailto = mp[3]

    return mailing_person(mailto, mp[0])


class Command(BaseCommand):
    help = "Send emails for all Signature for which it hasn't been done before."

    def handle(self, *args, **options):
        self.send_to_signatures()
        self.send_call_out_emails()

    def send_to_signatures(self):
        """Do the job."""
        start = time.time()

        # Because @async_to_sync doesn't update types
        # noinspection PyTypeChecker
        modified: List[Signature] = self.async_sent_to_signatures(
            list(Signature.get_unsent())
        )

        Signature.objects.bulk_update(modified, ("email_sent_at",))

        stop = time.time()
        self.stdout.write(
            f"Successfully sent {len(modified)} signatures emails "
            f"in {stop - start:.2}s."
        )

    @async_to_sync
    async def async_sent_to_signatures(
        self, signatures: Iterable[Signature]
    ) -> List[Signature]:
        """Send the mail to the signatures in an async way."""
        sem = Semaphore(MAX_CONCURRENT_REQUESTS)

        async with get_aiohttp_session() as session:
            results: List[SendSignature] = await asyncio.gather(
                *(
                    self.send_one_mail_to_a_signature(session, s, sem)
                    for s in signatures
                )
            )

        return [s for s in results if s is not None]

    async def send_one_mail_to_a_signature(
        self, session: ClientSession, signature: Signature, sem: Semaphore
    ) -> SendSignature:
        """
        Send an email to the corresponding signature.

        :param session: The client with which to send the email
        :param signature: the recipient of the email
        :param sem: A semaphore object to limit the number of concurrent requests
        :return: the signature, is sending was successful
        """
        name = signature.first_name + " " + signature.surname

        async with sem:
            sending_result = await send_mail(
                session=session,
                sender=MAIN_PERSON,
                to=(mailing_person(signature.email, name),),
                html_content="<p>Thank you for signing</p>",
                text_content="Thank you for signing",
                subject="Thank you for signin",
            )

        if sending_result:
            signature.email_sent_at = timezone.now()
            return signature

        else:
            self.stderr.write(f"Sending mail to {signature} failed.")
            return None

    def send_call_out_emails(self):
        """Send the pending emails in CallOutEmail table."""
        start = time.time()

        # noinspection PyTypeChecker
        modified: List[CallOutEmail] = self.async_call_out_emails(
            list(CallOutEmail.get_unsent())
        )

        CallOutEmail.objects.bulk_update(modified, ("email_sent_at",))

        stop = time.time()
        self.stdout.write(
            f"Successfully sent {len(modified)} call out emails in {stop - start:.2}s."
        )

    @async_to_sync
    async def async_call_out_emails(
        self, emails: List[CallOutEmail]
    ) -> List[CallOutEmail]:
        """Send the emails in an async way."""
        sem = Semaphore(MAX_CONCURRENT_REQUESTS)

        async with get_aiohttp_session() as session:
            result = await asyncio.gather(
                *(self.send_one_call_out_email(session, e, sem) for e in emails)
            )

        return [r for r in result if r is not None]

    async def send_one_call_out_email(
        self, session: ClientSession, email: CallOutEmail, sem: Semaphore
    ) -> SendCallOutEmail:
        """Send one call out email."""
        sender = mailing_person(MAIN_EMAIL, email.sender)
        to = [
            mailing_person_from_circonscription(email.circonscription_number),
        ]
        if email.send_to_government:
            to.append(get_gvt_address())
        cc = (mailing_person(email.from_email, email.sender),)
        html_content = "<p>" + email.content + "</p>"

        async with sem:
            sending_result = await send_mail(
                session,
                sender=sender,
                to=to,
                html_content=html_content,
                subject=email.subject,
                text_content=email.content,
                cc=cc,
            )

        if sending_result:
            email.email_sent_at = timezone.now()
            return email

        else:
            self.stderr.write(f"Sending mail to {email} failed.")
            return None



"""Real signature module."""

from django.core.exceptions import ValidationError
from django.http import (
    HttpRequest,
    HttpResponseBadRequest,
    HttpResponseNotFound,
    JsonResponse,
    HttpResponse,
)
from django.middleware import csrf
from django.shortcuts import redirect

from appel_crises.data import POSTAL_CODE_TO_DISTRICT, DISTRICT_TO_MP
from appel_crises.models import Signature, CallOutEmail


def sign(request: HttpRequest, is_ajax=False):
    """Main signature function"""
    # noinspection PyArgumentList
    try:
        first_name = request.POST['firstname']
        surname = request.POST['surname']
        email = request.POST['email']
    except KeyError as e:
        return HttpResponseBadRequest(f"Le champs {e} est requis.")

    signature = Signature(first_name=first_name, surname=surname, email=email)

    try:
        # Validate, among other things, the uniqueness of the email
        signature.full_clean()
    except ValidationError as e:
        # Probably a duplicate email, or a wrong email, or too long values
        return HttpResponseBadRequest("<br/>".join(e.messages))

    # I would put a try catch, but at this point i don't know what could go wrong
    signature.save()

    if is_ajax:
        return HttpResponse(status=200)
    else:
        return redirect('success')


def call_out_ajax(request: HttpRequest):
    """Form submission via an XHR request in the browser"""
    fields = [
        "content",
        "subject",
        "send_to_government",
        "circonscription_number",
        "sender",
        "from_email",
        "postal_code",
    ]
    try:
        values = {field: request.POST[field] for field in fields}
    except KeyError as e:
        return HttpResponseBadRequest(e)
    call_out_email = CallOutEmail(**values)
    call_out_email.send_to_government = int(call_out_email.send_to_government)

    try:
        call_out_email.full_clean()
    except ValidationError as e:
        # Probably a duplicate email, or a wrong email, or too long values
        return HttpResponseBadRequest(e)

    if call_out_email.circonscription_number not in DISTRICT_TO_MP:
        return HttpResponseBadRequest("Circonscription number not found")

    call_out_email.save()

    return HttpResponse(status=200)


def call_out(request: HttpRequest):
    """Form submission from no js page"""
    fields = [
        "content",
        "subject",
        "sender",
        "from_email",
        "postal_code",
    ]
    try:
        values = {field: request.POST[field] for field in fields}
    except KeyError as e:
        return HttpResponseBadRequest(e)
    call_out_email = CallOutEmail(**values)

    # noinspection PyCallByClass,PyTypeChecker
    if request.POST.get("send_to_government", "off") == 'on':
        call_out_email.send_to_government = 1
    else:
        call_out_email.send_to_government = False

    try:
        call_out_email.full_clean()
    except ValidationError as e:
        # Probably a duplicate email, or a wrong email, or too long values
        return HttpResponseBadRequest(e)

    # noinspection PyTypeChecker,PyCallByClass
    if request.POST.get("send_to_depute", False):
        postal_code = call_out_email.postal_code
        if postal_code not in POSTAL_CODE_TO_DISTRICT:
            return HttpResponseBadRequest("Postal code not found.")

        circo_number = POSTAL_CODE_TO_DISTRICT[postal_code][0]
        if circo_number not in DISTRICT_TO_MP:
            return HttpResponseBadRequest(
                "We don't know your circonscription. "
                "Please try and send him directly an email."
            )

        call_out_email.circonscription_number = circo_number

    call_out_email.save()

    return redirect("success-email")


def search_depute(request: HttpRequest):
    """Take input as POST parameter and return the corresponding depute as JSON"""

    postal_code = request.POST['postal_code']

    if postal_code not in POSTAL_CODE_TO_DISTRICT:
        return HttpResponseNotFound()

    districts = POSTAL_CODE_TO_DISTRICT[postal_code]

    mp_data = [(*DISTRICT_TO_MP[d], d) for d in districts if d in DISTRICT_TO_MP]

    # We have to pass safe=False, because we want to serialize an Array, see:
    # https://docs.djangoproject.com/en/3.0/ref/request-response/#serializing-non-dictionary-objects
    return JsonResponse(mp_data, safe=False)


def get_csrf_token(request: HttpRequest):
    """
    Return a CSRF token. Nothing big to do here, everything is done by the CSRF
    middleware.
    """
    return HttpResponse(csrf.get_token(request))



# Generated by Django 3.0 on 2020-04-20 12:57

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('appel_crises', '0002_auto_20200420_1244'),
    ]

    operations = [
        migrations.AlterField(
            model_name='signature',
            name='created',
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
    ]



import time
from argparse import ArgumentParser

from django.core.management import BaseCommand

from appel_crises.models import Signature

FAKE_ATTRIBUTES = dict(first_name="PURE", surname="Spammer",)


def generate_a_fake_signature(n: int) -> Signature:
    """Generate a fake signature based on integer n."""
    return Signature(email=f"spammer+f{n}f@dev.p-u-r-e.tech", **FAKE_ATTRIBUTES)


class Command(BaseCommand):
    help = (
        "Populate the database with fakes signatures or remove them from the database. "
    )

    def add(self, n: int):
        self.remove()
        signatures = (generate_a_fake_signature(i) for i in range(n))
        inserted = len(Signature.objects.bulk_create(signatures))

        return inserted

    def remove(self) -> int:
        removed, _ = Signature.objects.filter(**FAKE_ATTRIBUTES).delete()

        return removed

    def add_arguments(self, parser: ArgumentParser):
        group = parser.add_mutually_exclusive_group()
        group.add_argument(
            "-a", "--add", type=int, help="Add ADD entries to the database"
        )
        group.add_argument("-r", "--remove", action="store_true")

    def handle(self, *args, **options):
        if options['remove']:
            start = time.time()
            removed = self.remove()
            stop = time.time()
            self.stdout.write(
                f"Removed {removed} objects from the database in {stop - start:.2}s."
            )
        elif 'add' in options:
            start = time.time()
            inserted = self.add(options['add'])
            stop = time.time()
            self.stdout.write(
                f"Added {inserted} signatures into the databasein {stop - start:.2}s.\n"
                f"They looked like this :\n Signature"
                f"[{', '.join(k + '=' + v for k, v in FAKE_ATTRIBUTES.items())}]\n"
            )



ASSOCIATIONS = [
    ("https://pour-un-reveil-ecologique.org/", "Alter'Actions"),
    ("https://pour-un-reveil-ecologique.org/", "Animafac"),
    ("https://pour-un-reveil-ecologique.org/", "APF France Handicap"),
    (
        "https://pour-un-reveil-ecologique.org/",
        "Association nationale des étudiants en médecine de France",
    ),
    ("https://pour-un-reveil-ecologique.org/", "ATD-Quart Monde"),
    ("https://pour-un-reveil-ecologique.org/", "Avenir climatique"),
    (
        "https://pour-un-reveil-ecologique.org/",
        "B.A.-BA Fédération des associations étudiantes d'Aix-en-Provence",
    ),
    ("https://pour-un-reveil-ecologique.org/", "C3D"),
    ("https://pour-un-reveil-ecologique.org/", "Campus de la Transition"),
    ("https://pour-un-reveil-ecologique.org/", "Citoyens Pour Le Climat"),
    ("https://pour-un-reveil-ecologique.org/", "CliMates"),
    ("https://pour-un-reveil-ecologique.org/", "Colibris"),
    ("https://pour-un-reveil-ecologique.org/", "Comité 21"),
    ("https://pour-un-reveil-ecologique.org/", "Convergence Infirmière"),
    ("https://pour-un-reveil-ecologique.org/", "COP1"),
    ("https://pour-un-reveil-ecologique.org/", "COP2 étudiante"),
    ("https://pour-un-reveil-ecologique.org/", "Emmaus"),
    ("https://pour-un-reveil-ecologique.org/", "Entrepreneurs d'Avenir"),
    ("https://pour-un-reveil-ecologique.org/", "FEF"),
    ("https://pour-un-reveil-ecologique.org/", "FFJ"),
    ("https://pour-un-reveil-ecologique.org/", "Filière Paysanne "),
    ("https://pour-un-reveil-ecologique.org/", "FNE"),
    ("https://pour-un-reveil-ecologique.org/", "Fondation Nicolas Hulot"),
    ("https://pour-un-reveil-ecologique.org/", "Gilets Citoyens"),
    ("https://pour-un-reveil-ecologique.org/", "Greenlobby"),
    ("https://pour-un-reveil-ecologique.org/", "Groupe SOS Transition écologique"),
    ("https://pour-un-reveil-ecologique.org/", "I4CE"),
    ("https://pour-un-reveil-ecologique.org/", "Il est encore temps"),
    ("https://pour-un-reveil-ecologique.org/", "Institut Rousseau"),
    ("https://pour-un-reveil-ecologique.org/", "Institut Veblen"),
    ("https://pour-un-reveil-ecologique.org/", "La Fresque du climat"),
    ("https://pour-un-reveil-ecologique.org/", "Latitudes"),
    ("https://pour-un-reveil-ecologique.org/", "Les greniers d'abondance"),
    ("https://pour-un-reveil-ecologique.org/", "Les Shifters"),
    ("https://pour-un-reveil-ecologique.org/", "Lobby Citoyen (CLIC)"),
    ("https://pour-un-reveil-ecologique.org/", "Low Carbon France"),
    ("https://pour-un-reveil-ecologique.org/", "Lumia"),
    ("https://pour-un-reveil-ecologique.org/", "Négawatt"),
    ("https://pour-un-reveil-ecologique.org/", "NOISE"),
    ("https://pour-un-reveil-ecologique.org/", "Notre Affaire à Tous"),
    ("https://pour-un-reveil-ecologique.org/", "On est prêt"),
    ("https://pour-un-reveil-ecologique.org/", "Paris-Bascule"),
    ("https://pour-un-reveil-ecologique.org/", "Pour un réveil écologique"),
    ("https://pour-un-reveil-ecologique.org/", "Printemps écologique"),
    ("https://pour-un-reveil-ecologique.org/", "Profs en Transition"),
    ("https://pour-un-reveil-ecologique.org/", "REFEDD"),
    ("https://pour-un-reveil-ecologique.org/", "Resilience"),
    ("https://pour-un-reveil-ecologique.org/", "Sciences Citoyennes"),
    ("https://pour-un-reveil-ecologique.org/", "SOS Maires"),
    ("https://pour-un-reveil-ecologique.org/", "T4E/Up for Europe"),
    ("https://pour-un-reveil-ecologique.org/", "The Shift Project"),
    ("https://pour-un-reveil-ecologique.org/", "UNICEF"),
    ("https://pour-un-reveil-ecologique.org/", "Unis pour la planète"),
    ("https://pour-un-reveil-ecologique.org/", "URIOPSS Nouvelle-Aquitaine"),
    ("https://pour-un-reveil-ecologique.org/", "WARN"),
    ("https://pour-un-reveil-ecologique.org/", "WWF"),
    ("https://pour-un-reveil-ecologique.org/", "Yon A Lot - L'un à l'autre"),
    ("https://pour-un-reveil-ecologique.org/", "Youth For Climate France"),
]



"""
Django settings for appel_crises project.

Generated by 'django-admin startproject' using Django 3.0.5.

For more information on this file, see
https://docs.djangoproject.com/en/3.0/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.0/ref/settings/
"""

import os

import getconf
import sentry_sdk
from sentry_sdk.integrations.django import DjangoIntegration

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

config = getconf.ConfigGetter(
    'appel_crises',
    ['/etc/appel_crises/*.ini', os.path.join(BASE_DIR, 'local_settings.ini')],
)

APPMODE = config.getstr('app.mode', 'dev')
assert APPMODE in ('dev', 'dist', 'prod'), "Invalid application mode %s" % APPMODE

if APPMODE in ('dev', 'dist'):
    _default_secret_key = 'Dev only!!'
else:
    _default_secret_key = ''

SECRET_KEY = config.getstr('app.secret_key', _default_secret_key)

DEBUG = config.getbool('app.debug', APPMODE == 'dev')
TEMPLATE_DEBUG = DEBUG

ALLOWED_HOSTS = config.getlist('site.allowed_hosts')


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'appel_crises',
]

if APPMODE != 'prod':
    INSTALLED_APPS += [
        # Run with whitenoise in development too. Avoids issues with
        # invalid {% static '/appel_crises/...' %} that fail in production.
        'whitenoise.runserver_nostatic',
    ]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'whitenoise.middleware.WhiteNoiseMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'appel_crises.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'appel_crises.wsgi.application'

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {'console': {'class': 'logging.StreamHandler'}},
    'root': {'handlers': ['console'], 'level': 'INFO'},
}

psql_db = {
    'ENGINE': 'django.db.backends.postgresql',
    'HOST': config.getstr('db.host', 'localhost'),
    'NAME': config.getstr('db.name'),
    'PASSWORD': config.getstr("db.password"),
    'PORT': config.getstr('db.port', ''),
    'USER': config.getstr("db.user"),
}
sqlite_db = {
    'ENGINE': 'django.db.backends.sqlite3',
    'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
}

DATABASES = {
    'default': sqlite_db if config.getstr("db.database") == "sqlite" else psql_db
}


# Password validation
# https://docs.djangoproject.com/en/3.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation'
        '.UserAttributeSimilarityValidator',
    },
    {'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator'},
    {'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator'},
    {'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator'},
]


# Internationalization
# https://docs.djangoproject.com/en/3.0/topics/i18n/

LANGUAGE_CODE = config.getstr('i18n.language', 'fr-FR')

TIME_ZONE = config.getstr('i18n.tz', 'Europe/Paris')

USE_I18N = True
USE_L10N = True
USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.0/howto/static-files/

STATIC_URL = config.getstr('site.assets_url', '/assets/')
STATIC_ROOT = os.path.join(BASE_DIR, 'appel_crises', 'assets')
STATICFILES_STORAGE = 'whitenoise.storage.CompressedManifestStaticFilesStorage'

# SendinBlue Settings
SENDIN_BLUE_API_KEY = config.getstr('mail.sendinblue_api_key')
MAIN_EMAIL = config.getstr('mail.main_email')
MAIN_NAME = config.getstr('mail.main_name')

# Sentry settings
if not DEBUG:
    sentry_sdk.init(
        dsn=config.getstr('sentry.api_key'), integrations=[DjangoIntegration()],
    )



"""Python module that manages the emails for the app.

It uses SendinBlue API as documented here : https://developers.sendinblue.com/reference

"""
import logging
from datetime import datetime
from typing import Dict, Iterable, Tuple, List

from aiohttp import ClientSession
from asgiref.sync import async_to_sync

from appel_crises.settings import SENDIN_BLUE_API_KEY, MAIN_EMAIL, MAIN_NAME

SB_SEND_MAIL_URL = "https://api.sendinblue.com/v3/smtp/email"
SB_FETCH_EVENTS_URL = "https://api.sendinblue.com/v3/smtp/statistics/events"

_logger = logging.getLogger(__name__)

MailingPerson = Dict[str, str]
DeliveryResult = Tuple[str, datetime]


def mailing_person(email: str, name: str = None) -> MailingPerson:
    """Build a MailingPerson dict."""
    if name is not None:
        return dict(name=name, email=email)
    else:
        return dict(email=email)


MAIN_PERSON = mailing_person(MAIN_EMAIL, MAIN_NAME)


def get_aiohttp_session() -> ClientSession:
    """Open session"""
    return ClientSession(headers={'api-key': SENDIN_BLUE_API_KEY})


async def send_mail(
    session: ClientSession,
    sender: MailingPerson,
    to: Iterable[MailingPerson],
    html_content: str,
    subject: str,
    text_content: str = "",
    bcc: Iterable[MailingPerson] = None,
    cc: Iterable[MailingPerson] = None,
) -> bool:
    """Uses SendinBlue API to send an email"""

    body = dict(
        sender=sender,
        to=to,
        htmlContent=html_content,
        textContent=text_content,
        subject=subject,
    )

    if bcc is not None:
        body['bcc'] = bcc

    if cc is not None:
        body['cc'] = cc

    async with session.post(SB_SEND_MAIL_URL, json=body) as resp:
        if resp.status == 201:
            return True
        else:
            _logger.error(
                "Invalid response %d on POST to send mail: %s",
                resp.status,
                await resp.text(),
            )
            return False


@async_to_sync
async def get_delivered_emails(offset: int, limit: int = 100) -> List[DeliveryResult]:
    """Fetch the delivered emails."""
    params = dict(offset=offset, limit=limit, event="delivered")

    async with get_aiohttp_session() as session:
        async with session.get(SB_FETCH_EVENTS_URL, params=params) as resp:
            data = await resp.json()

    if not data or 'events' not in data:
        return []

    result = [(e['email'], datetime.fromisoformat(e['date'])) for e in data['events']]

    return result



import random
import string
import time
from tqdm import tqdm

from appel_crises.models import Signature

LETTERS = string.ascii_lowercase


def random_letters(n):
    return random.sample(LETTERS, n)


def generate_random_signature_data():
    first_name = random_letters(5)
    surname = random_letters(5)
    email = f"{random_letters(8)}@{random_letters(7)}.{random_letters(3)}"
    return email, first_name, surname


def insert_signatures(n):
    insert_times = []
    for _ in tqdm(range(n)):
        email, first_name, surname = generate_random_signature_data()
        Signature(email=email, first_name=first_name, surname=surname).save()
        insert_times.append(time.time())

    return insert_times



from typing import List

from appel_crises.mailer import MailingPerson, mailing_person

GVT: List[MailingPerson] = [
    mailing_person("hadrien.renaudlebret+0@gmail.com", "Emmanuel Macron"),
    mailing_person("hadrien.renaudlebret+1@gmail.com", "Emmanuel Macron"),
    mailing_person("hadrien.renaudlebret+2@gmail.com", "Emmanuel Macron"),
    mailing_person("hadrien.renaudlebret+3@gmail.com", "Emmanuel Macron"),
    mailing_person("hadrien.renaudlebret+4@gmail.com", "Emmanuel Macron"),
    mailing_person("hadrien.renaudlebret+5@gmail.com", "Emmanuel Macron"),
    mailing_person("hadrien.renaudlebret+6@gmail.com", "Emmanuel Macron"),
    mailing_person("hadrien.renaudlebret+7@gmail.com", "Emmanuel Macron"),
    mailing_person("hadrien.renaudlebret+8@gmail.com", "Emmanuel Macron"),
    mailing_person("hadrien.renaudlebret+9@gmail.com", "Emmanuel Macron"),
    mailing_person("hadrien.renaudlebret+10@gmail.com", "Emmanuel Macron"),
    mailing_person("hadrien.renaudlebret+11@gmail.com", "Emmanuel Macron"),
    mailing_person("hadrien.renaudlebret+12@gmail.com", "Emmanuel Macron"),
    mailing_person("hadrien.renaudlebret+13@gmail.com", "Emmanuel Macron"),
    mailing_person("hadrien.renaudlebret+14@gmail.com", "Emmanuel Macron"),
    mailing_person("hadrien.renaudlebret+15@gmail.com", "Emmanuel Macron"),
    mailing_person("hadrien.renaudlebret+16@gmail.com", "Emmanuel Macron"),
    mailing_person("hadrien.renaudlebret+17@gmail.com", "Emmanuel Macron"),
    mailing_person("hadrien.renaudlebret+18@gmail.com", "Emmanuel Macron"),
    mailing_person("hadrien.renaudlebret+19@gmail.com", "Emmanuel Macron"),
    mailing_person("hadrien.renaudlebret+20@gmail.com", "Emmanuel Macron"),
]



# Generated by Django 3.0 on 2020-04-20 12:44

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('appel_crises', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='signature',
            name='email',
            field=models.EmailField(max_length=100, unique=True),
        ),
    ]



# Generated by Django 3.0.5 on 2020-04-28 20:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('appel_crises', '0007_emailoffset'),
    ]

    operations = [
        migrations.AddField(
            model_name='signature',
            name='from_asso',
            field=models.CharField(max_length=40, null=True),
        ),
    ]



"""Module that provides the views of the app."""
from django.http import HttpRequest
from django.shortcuts import render

from appel_crises.constants import ASSOCIATIONS
from appel_crises.models import Signature


def home(request: HttpRequest):
    """Home view."""
    signatures = '{:,}'.format(Signature.get_count()).replace(',', ' ')
    context = dict(counter=signatures, associations=ASSOCIATIONS,)
    return render(request, "home.html", context=context)


def success(request: HttpRequest):
    """Signature success page"""
    return render(request, "success.html")


def success_email(request: HttpRequest):
    """Signature success page"""
    return render(request, "success_email.html")


def form_no_js(request: HttpRequest):
    """Special page without js for signature"""
    return render(request, "form_no_js.html")


def call_out_no_js(request: HttpRequest):
    """Special page without js for call_out"""
    return render(request, "call_out_no_js.html")



"""appel_crises URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/3.0/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from functools import partial

from django.contrib import admin
from django.urls import path

from appel_crises.api import (
    sign,
    search_depute,
    get_csrf_token,
    call_out_ajax,
    call_out,
)
from appel_crises.views import home, success, form_no_js, success_email, call_out_no_js

urlpatterns = [
    path('admin/', admin.site.urls),
    path('success', success, name='success'),
    path('success-email', success_email, name='success-email'),
    path('api/call-out', call_out, name='call-out'),
    path('api/call-out-ajax', call_out_ajax, name='call-out-ajaxs'),
    path('api/sign', sign, name='sign'),
    path('api/sign-ajax', partial(sign, is_ajax=True), name='sign-ajax'),
    path('api/search-depute', search_depute, name='search-depute'),
    path('api/csrf', get_csrf_token, name='get-csrf'),
    path('', home),
    path('form', form_no_js, name="form-no-js"),
    path('call-out', call_out_no_js, name="call-out-no-js"),
]



# Generated by Django 3.0.5 on 2020-04-29 19:52

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('appel_crises', '0008_signature_from_asso'),
    ]

    operations = [
        migrations.CreateModel(
            name='CallOutEmail',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(default=django.utils.timezone.now)),
                ('content', models.TextField()),
                ('subject', models.CharField(max_length=200)),
                ('send_to_government', models.BooleanField()),
                ('send_to_depute', models.BooleanField()),
                ('sender', models.CharField(max_length=100)),
                ('from_email', models.CharField(max_length=60)),
                ('postal_code', models.CharField(max_length=10)),
                ('email_sent_at', models.DateTimeField(blank=True, db_index=True, null=True)),
                ('verified_at', models.DateTimeField(blank=True, db_index=True, null=True)),
                ('from_asso', models.CharField(blank=True, max_length=40, null=True)),
            ],
        ),
    ]



"""
Data module. This little python file only imports the json stored in the directory.
"""
import json
import os
from typing import Tuple, Dict, List

from appel_crises.settings import BASE_DIR


def load_json_file(filename: str):
    """Load from json file in this directory"""
    with open(os.path.join(BASE_DIR, "appel_crises", "data", filename)) as f:
        return json.load(f)


DISTRICT_TO_MP: Dict[str, Tuple[str, str, str, str]] = load_json_file(
    "district_to_mp.json"
)

POSTAL_CODE_TO_DISTRICT: Dict[str, List[str]] = load_json_file(
    "postal_code_to_district.json"
)



from django.contrib import admin

from appel_crises.models import Signature, CallOutEmail


@admin.register(Signature)
class SignatureAdmin(admin.ModelAdmin):
    list_display = (
        'email',
        'surname',
        'first_name',
        'created_at',
        'email_sent_at',
        'verified_at',
    )

    def has_change_permission(self, request, obj=None):
        return False

    def has_add_permission(self, request):
        return False


@admin.register(CallOutEmail)
class CallOutEmailAdmin(admin.ModelAdmin):
    list_display = (
        'from_email',
        'sender',
        'postal_code',
        'circonscription_number',
        'send_to_government',
        'created_at',
        'email_sent_at',
        'verified_at',
    )

    def has_change_permission(self, request, obj=None):
        return False

    def has_add_permission(self, request):
        return False
